@@ -1,22 +1,32 @@
-#  Low level wrapper for odbc driver
+#  Low level wrappers for odbc driver
 #
 #
 #
-.First.lib <- function(lib, pkg)
+.onLoad <- function(lib, pkg)
 {
     library.dynam("RODBC", pkg, lib)
-    .Call("RODBCInit", PACKAGE = "RODBC")
-    if(is.null(getOption("dec"))) options(dec = ".")
+    if(is.null(getOption("dec")))
+        options(dec = Sys.localeconv()$decimal_point)
+}
+
+.onAttach <- function(lib, pkg)
+{
+    unlockBinding("typesR2DBMS", asNamespace("RODBC"))
+}
+
+.onUnload <- function(libpath)
+{
+    odbcCloseAll()
+    library.dynam.unload("RODBC", libpath)
 }
 
 odbcGetErrMsg <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-#    num <- .Call("RODBCErrMsgCount", channel, PACKAGE = "RODBC")
-#    if(num == 0) return(invisible(0))
-    err <- .Call("RODBCGetErrMsg", channel, PACKAGE = "RODBC")
-    .Call("RODBCClearError", as.integer(channel), PACKAGE = "RODBC")
+    err <- .Call("RODBCGetErrMsg", attr(channel, "handle_ptr"),
+                 PACKAGE = "RODBC")
+    .Call("RODBCClearError", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
     return(err)
 }
 
@@ -24,14 +34,14 @@ odbcClearError <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCClearError", channel, PACKAGE = "RODBC")
+    .Call("RODBCClearError", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
     invisible()
 }
 
 odbcReConnect <- function(channel, case, believeNRows)
 {
     if(!inherits(channel, "RODBC"))
-        stop("Argument `channel' must inherit from class RODBC")
+        stop("Argument 'channel' must inherit from class RODBC")
     if(missing(case)) case <- attr(channel, "case")
     if(missing(believeNRows)) believeNRows <- attr(channel, "believeNRows")
     odbcDriverConnect(attr(channel, "connection.string"), case, believeNRows)
@@ -41,12 +51,14 @@ odbcConnect <-
     function (dsn, uid = "", pwd = "", case = "nochange",
               believeNRows = TRUE)
 {
-   odbcDriverConnect(paste("DSN=", dsn, ";UID=",uid, ";PWD=", pwd, sep=""),
-                     case = case, believeNRows = believeNRows)
+    st <- paste("DSN=", dsn, sep="")
+    if(nchar(uid)) st <- paste(st, ";UID=", uid, sep="")
+    if(nchar(pwd)) st <- paste(st, ";PWD=", pwd, sep="")
+    odbcDriverConnect(st, case = case, believeNRows = believeNRows)
 }
 
 odbcDriverConnect <-
-    function (connection, case = "nochange", believeNRows = TRUE)
+    function (connection = "", case = "nochange", believeNRows = TRUE)
 {
    switch(case,
 	toupper = case <- 1,
@@ -74,11 +86,12 @@ odbcQuery <- function(channel, query)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCQuery", channel, as.character(query), PACKAGE = "RODBC")
+    .Call("RODBCQuery", attr(channel, "handle_ptr"), as.character(query),
+          PACKAGE = "RODBC")
 }
 
 odbcUpdate <-
-    function(channel, query, data, names, test = FALSE, verbose = FALSE,
+    function(channel, query, data, params, test = FALSE, verbose = FALSE,
              nastring = NULL)
 {
     if(!odbcValidChannel(channel))
@@ -86,8 +99,8 @@ odbcUpdate <-
     vflag <- 0
     if(verbose) vflag <- 1
     if(test) vflag <- 2
-#apply the name mangling that was applied when the table was created
-    cnames<- gsub("[^A-Za-z0-9]+", "", as.character(colnames(data)))
+    ## apply the name mangling that was applied when the table was created
+    cnames<- gsub("[^A-Za-z0-9_]+", "", as.character(colnames(data)))
     cnames <- switch(attr(channel, "case"),
                      nochange = cnames,
                      toupper = toupper(cnames),
@@ -95,37 +108,31 @@ odbcUpdate <-
     for(i in 1:ncol(data))
         if(!is.numeric(data[[i]]))
            data[[i]] <- as.character(data[[i]])
-    .Call("RODBCUpdate", as.integer(channel), as.character(query),
+    .Call("RODBCUpdate", attr(channel, "handle_ptr"), as.character(query),
           data, cnames, as.integer(nrow(data)), as.integer(ncol(data)),
-          as.character(names), as.integer(vflag),  PACKAGE = "RODBC")
+          as.character(params), as.integer(vflag), PACKAGE = "RODBC")
 }
 
 odbcTables <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCTables", channel, PACKAGE = "RODBC")
+    .Call("RODBCTables", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
 }
 
-odbcTypeInfo <- function(channel, type)
-{
-    if(!odbcValidChannel(channel))
-       stop("first argument is not an open RODBC channel")
-    .Call("RODBCTypeInfo", channel, as.integer(type), PACKAGE = "RODBC")
- }
-
 odbcColumns <- function(channel, table)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCColumns", channel, as.character(table), PACKAGE = "RODBC")
+    .Call("RODBCColumns", attr(channel, "handle_ptr"), as.character(table),
+          PACKAGE = "RODBC")
 }
 
 odbcSpecialColumns <- function(channel, table)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCSpecialColumns", channel, as.character(table),
+    .Call("RODBCSpecialColumns", attr(channel, "handle_ptr"), as.character(table),
           PACKAGE = "RODBC")
 }
 
@@ -133,7 +140,8 @@ odbcPrimaryKeys <- function(channel, table)
 {
     if(!odbcValidChannel(channel))
         stop("first argument is not an open RODBC channel")
-    .Call("RODBCPrimaryKeys", channel, as.character(table), PACKAGE = "RODBC")
+    .Call("RODBCPrimaryKeys", attr(channel, "handle_ptr"), as.character(table),
+          PACKAGE = "RODBC")
 }
 
 ## this does no error checking, but may add an error message
@@ -141,22 +149,14 @@ odbcColData <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCColData", channel, PACKAGE = "RODBC")
-}
-
-## currently unused
-odbcNumRows <- function(channel)
-{
-    if(!odbcValidChannel(channel))
-       stop("first argument is not an open RODBC channel")
-    .Call("RODBCNumRows", channel, PACKAGE = "RODBC")
+    .Call("RODBCColData", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
 }
 
 odbcNumCols <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCNumCols", channel, PACKAGE = "RODBC")
+    .Call("RODBCNumCols", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
 }
 
 close.RODBC <- function(con, ...) odbcClose(con)
@@ -164,8 +164,8 @@ close.RODBC <- function(con, ...) odbcClose(con)
 odbcClose <- function(channel)
 {
     if(!odbcValidChannel(channel))
-       stop("first argument is not an open RODBC channel")
-    res <- .Call("RODBCClose", channel, PACKAGE = "RODBC")
+       stop("argument is not an open RODBC channel")
+    res <- .Call("RODBCClose", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
     if(res > 0) invisible(TRUE) else {
         warning(paste(odbcGetErrMsg(channel), sep="\n"))
         FALSE
@@ -184,7 +184,7 @@ odbcFetchRows <-
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCFetchRows", channel, max, buffsize,
+    .Call("RODBCFetchRows", attr(channel, "handle_ptr"), max, buffsize,
           as.character(nullstring), believeNRows, PACKAGE = "RODBC")
 }
 
@@ -199,7 +199,11 @@ odbcGetInfo <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCGetInfo", channel, PACKAGE = "RODBC")
+    res <- .Call("RODBCGetInfo", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    names(res) <- c("DBMS_Name", "DBMS_Ver", "Driver_ODBC_Ver",
+                    "Data_Source_Name", "Driver_Name", "Driver_Ver",
+                    "ODBC_Ver", "Server_Name")
+    res
 }
 
 odbcValidChannel <-  function(channel)
@@ -213,7 +217,7 @@ odbcClearResults <-  function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCclearresults", channel, PACKAGE = "RODBC")
+    .Call("RODBCclearresults", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
     invisible()
 }
 
